//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * 二要素実数位置ヘッダ
 * @author Junpee
 */

#ifndef POINT_2F_H_
#define POINT_2F_H_

namespace Lamp{

class Point2i;

//------------------------------------------------------------------------------
/**
 * 二要素実数位置
 *
 * このクラスは継承しないで下さい。
 */
class Point2f{
public:
	//--------------------------------------------------------------------------
	// メンバ変数
	//--------------------------------------------------------------------------
	/// メンバ変数
	union{
		/// 各要素
		struct{
			/// X値
			float x;
			/// Y値
			float y;
		};

		/// 配列
		float array[2];
	};

	//--------------------------------------------------------------------------
	// 定数
	//--------------------------------------------------------------------------
	/// ゼロ位置
	static const Point2f zero;

	/// 単位位置
	static const Point2f unit;

	//--------------------------------------------------------------------------
	// コンストラクタ
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 *
	 * このコンストラクタは初期値の設定を行わないため値は不定です。
	 */
	Point2f(){}

	/**
	 * コンストラクタ
	 * @param sourceX X値の初期値
	 * @param sourceY Y値の初期値
	 */
	Point2f(float sourceX, float sourceY) : x(sourceX), y(sourceY){}

	/**
	 * コンストラクタ
	 * @param sourceArray 初期値配列
	 */
	explicit Point2f(float sourceArray[2]) :
		x(sourceArray[0]), y(sourceArray[1]){}

	/**
	 * コンストラクタ
	 * @param source 初期化する位置
	 */
	explicit Point2f(const Point2i& source);

	//--------------------------------------------------------------------------
	// 値の設定
	//--------------------------------------------------------------------------
	/**
	 * 値の設定
	 * @param sourceX X値の設定値
	 * @param sourceY Y値の設定値
	 */
	inline void set(float sourceX, float sourceY){
		x = sourceX;
		y = sourceY;
	}

	/**
	 * 値の設定
	 * @param sourceArray 設定値配列
	 */
	inline void set(float sourceArray[2]){
		x = sourceArray[0];
		y = sourceArray[1];
	}

	/**
	 * 値の設定
	 * @param source 設定する位置
	 */
	void set(const Point2i& source);

	//--------------------------------------------------------------------------
	// 演算
	//--------------------------------------------------------------------------
	/**
	 * 加算
	 * @param addPoint 加算する位置
	 * @return 加算された位置
	 */
	inline Point2f operator +(const Point2f& addPoint) const{
		return Point2f(x + addPoint.x, y + addPoint.y);
	}

	/**
	 * 減算
	 * @param subPoint 減算する位置
	 * @return 減算された位置
	 */
	inline Point2f operator -(const Point2f& subPoint) const{
		return Point2f(x - subPoint.x, y - subPoint.y);
	}

	/**
	 * 乗算
	 * @param mulPoint 乗算する位置
	 * @return 乗算された位置
	 */
	inline Point2f operator *(const Point2f& mulPoint) const{
		return Point2f(x * mulPoint.x, y * mulPoint.y);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された位置
	 */
	inline Point2f operator *(float mulValue) const{
		return Point2f(x * mulValue, y * mulValue);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @param mulPoint 乗算される位置
	 * @return 乗算された位置
	 */
	inline friend Point2f operator *(float mulValue, const Point2f& mulPoint){
		return Point2f(mulPoint.x * mulValue, mulPoint.y * mulValue);
	}

	/**
	 * +演算子
	 * @return 位置のコピー
	 */
	inline Point2f operator +() const{ return *this; }

	/**
	 * -演算子
	 * @return 値の符号が反転した位置
	 */
	inline Point2f operator -() const{ return Point2f(-x, -y); }

	//--------------------------------------------------------------------------
	// 代入演算
	//--------------------------------------------------------------------------
	/**
	 * 代入加算
	 * @param addPoint 加算する位置
	 * @return 加算された位置
	 */
	inline Point2f& operator +=(const Point2f& addPoint){
		x += addPoint.x;
		y += addPoint.y;
		return (*this);
	}

	/**
	 * 代入減算
	 * @param subPoint 減算する位置
	 * @return 減算された位置
	 */
	inline Point2f& operator -=(const Point2f& subPoint){
		x -= subPoint.x;
		y -= subPoint.y;
		return (*this);
	}

	/**
	 * 代入乗算
	 * @param mulPoint 乗算する位置
	 * @return 乗算された位置
	 */
	inline Point2f& operator *=(const Point2f& mulPoint){
		x *= mulPoint.x;
		y *= mulPoint.y;
		return (*this);
	}

	/**
	 * 代入乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された位置
	 */
	inline Point2f& operator *=(float mulValue){
		x = x * mulValue;
		y = y * mulValue;
		return (*this);
	}

	//--------------------------------------------------------------------------
	// 論理演算
	//--------------------------------------------------------------------------
	/**
	 * 同じ値かどうか
	 * @param target 比較する位置
	 * @return 同じ値であればtrueを返す
	 */
	inline bool operator ==(const Point2f& target) const{
		return ((x == target.x) && (y == target.y));
	}

	/**
	 * 同じ値でないかどうか
	 * @param target 比較する位置
	 * @return 同じ値でなければtrueを返す
	 */
	inline bool operator !=(const Point2f& target) const{
		return ((x != target.x) || (y != target.y));
	}

	//--------------------------------------------------------------------------
	// その他
	//--------------------------------------------------------------------------
	/**
	 * 文字列化
	 * @return 位置の文字列表記
	 */
	inline String toString() const{
		String returnString;
		returnString.format("( %.8f, %.8f )", x, y);
		return returnString;
	}

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of POINT_2F_H_
//------------------------------------------------------------------------------
